/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 06 - Usando Cursores Explcitos.

Falar sobre cursores  abordar um assunto polmico... J que a maioria dos desenvolvedores no sabem quando utiliz-los.

Por isso antes de voc fazer como eu, que at poucos meses criava cursores para praticamente tudo, voc deve lembrar que
cursores (explcitos) consomem muitos recursos do servidor, facilitando a ocorrncia de deadlocks que poderiam ser 
evitados.

Vamos falar sobre um pouquinho de teoria antes do exemplo prtico...

Primeiro, vamos definir um cursor como um arquivo temporrio que armazena e controla as linhas retornadas de um comando 
select... Creio que esta definio gere algumas polmicas, mas para a minha explicao esta definio ajudar bastante.

No Oracle, temos dois tipos de cursores os cursores implcitos e os cursores explcitos. 

Vamos comear pelo mais simples deles, que voc j usou milhares de vezes, que  o cursor implcito.

Todo comando de Insert, Delete, Update e Select (QUANDO RETORNA APENAS UMA LINHA)  tratado pelo Oracle como um cursor 
implcito.

Como o intuito deste artigo no  falar de todos os comandos DML e DDL, estaremos focados nos cursores explcitos!

Estes cursores precisam ser declarados e os dados retornados da consulta so acessados atravs de variveis.

Vamos dividir a criao de cursores em quatro passos bsicos:

1.       Declarao;

2.       Abertura;

3.       Atribuio dos valores s variveis;

4.       Fechamento.

Vou resumir o que acontece em cada uma das etapas...

Na declarao do cursor atribumos um nome a ele, e um comando SELECT. Quando este cursor  aberto, o comando select  
executado. Com o cursor aberto podemos atribuir valores retornados pelo select s variveis, manipul-las e ento 
fechar o cursor para liberar os recursos do servidor.

Voc pode criar diversos cursores, mas deve estar ciente que  preciso tomar cuidado com a memria alocada para eles.

Outra coisa bem legal sobre os cursores  que eles podem receber parmetros, e estes parmetros podem ser inicializados 
com um valor default

Cursores e bulk collect, que nada mais so que matrizes de dados que armazenam os resultados de comandos select sendo 
comparado a um recordset.

O Bulk collect tem basicamente o mesmo conceito de um cursor, porm, com a vantagem de ser tratado como um array de dados, 
podendo ser manipulado dentro de um loop simples.

Facilidade de Implementao

No ponto de vista de implementao dos cursores, o FOR  os mais simples, bastando ser declarado ao longo do programa. 
Os cursores de BULK COLLECT vm segundo lugar, pois necessitam que sejam declarados variveis baseadas em types, 
tornando a implementao pouca coisa mais trabalhosa. J os cursores de FETCH vm em ltimo lugar. Como devem ser 
declarados juntamente com as variveis que recebero os valores, alm de que devem abertos ao longo da execuo do 
programa e fechados, sua implementao torna-se mais trabalhosa.

BULK COLLECT Forma menos usual, alguns front end no suportam este tipo de cursor, que  o caso do FORMS.
 
 

-- implicito
SET SERVEROUTPUT ON
BEGIN
    UPDATE    HR.EMPLOYEES
    SET       FIRST_NAME = FIRST_NAME
    WHERE     ROWNUM = 1;
    
    -- se commitar nao consegue mais pegar os atributos abaixo.
    
    IF SQL%FOUND THEN -- foi atualizado alguma linha 
      DBMS_OUTPUT.PUT_LINE('Foi(ram) atualizada(s) ' || SQL%ROWCOUNT || ' linha(s).');
    ELSE
      DBMS_OUTPUT.PUT_LINE('Nenhuma linha foi atualizada');
    END IF;
END;

-- cursor normal LOOP
SET SERVEROUTPUT ON
DECLARE
  
  V_EMP_ID HR.EMPLOYEES.EMPLOYEE_ID%TYPE;
  V_EMP_FN HR.EMPLOYEES.FIRST_NAME%TYPE;
  V_EMP_LN HR.EMPLOYEES.LAST_NAME%TYPE;
  
  CURSOR EMP_CUR IS 
      SELECT  EMPLOYEE_ID, FIRST_NAME, LAST_NAME
      FROM    HR.EMPLOYEES
      WHERE   ROWNUM<5;
      
BEGIN

  OPEN EMP_CUR;  
  
      IF EMP_CUR%ISOPEN THEN
        LOOP
            FETCH EMP_CUR INTO V_EMP_ID, V_EMP_FN, V_EMP_LN;
            EXIT WHEN EMP_CUR%NOTFOUND;
       
              DBMS_OUTPUT.PUT_LINE('Nome do empregado com ID ' || V_EMP_ID || ': ' || V_EMP_FN || ' ' || V_EMP_LN);
          
        END LOOP;
      END IF;
  
  CLOSE EMP_CUR;
END;

-- cursor for loop
SET SERVEROUTPUT ON
DECLARE
  CURSOR EMP_CUR IS 
      SELECT  EMPLOYEE_ID, FIRST_NAME, LAST_NAME
      FROM    HR.EMPLOYEES
      WHERE   ROWNUM<5;  
BEGIN
  FOR EMP_LINHA IN EMP_CUR 
  LOOP    
    DBMS_OUTPUT.PUT_LINE('Nome do empregado com ID ' || emp_linha.EMPLOYEE_ID || ': ' || emp_linha.FIRST_NAME || ' ' || emp_linha.LAST_NAME);
  END LOOP; 
END;

--a partir do 10g esse  o mais performatico


-- cursor for loop sem declarao
SET SERVEROUTPUT ON
BEGIN
  FOR EMP_LINHA IN (SELECT  EMPLOYEE_ID, FIRST_NAME, LAST_NAME
                    FROM    HR.EMPLOYEES
                    WHERE   ROWNUM<5) 
  LOOP    
    DBMS_OUTPUT.PUT_LINE('Nome do empregado com ID ' || emp_linha.EMPLOYEE_ID || ': ' || emp_linha.FIRST_NAME || ' ' || emp_linha.LAST_NAME);
  END LOOP; 
END;

-- nao precisa mais fazer cursor
-- mais rapido...


-- cursor com parametros
SET SERVEROUTPUT ON
DECLARE
  CURSOR EMP_CUR(P_EMP_ID NUMBER) IS 
      SELECT  EMPLOYEE_ID, FIRST_NAME, LAST_NAME
      FROM    HR.EMPLOYEES
      WHERE   EMPLOYEE_ID=P_EMP_ID;  
  
  V_EMPID HR.EMPLOYEES.EMPLOYEE_ID%TYPE;
  V_FN HR.EMPLOYEES.FIRST_NAME%TYPE;
  V_LN HR.EMPLOYEES.LAST_NAME%TYPE;  
BEGIN
 OPEN EMP_CUR(100); 
 FETCH EMP_CUR INTO V_EMPID, V_FN, V_LN;
 DBMS_OUTPUT.PUT_LINE('Nome do empregado com ID ' || V_EMPID || ': ' || V_FN || ' ' || V_LN);
 CLOSE EMP_CUR;
 
 OPEN EMP_CUR(103); 
 FETCH EMP_CUR INTO V_EMPID, V_FN, V_LN;
 DBMS_OUTPUT.PUT_LINE('Nome do empregado com ID ' || V_EMPID || ': ' || V_FN || ' ' || V_LN);
 CLOSE EMP_CUR;
END;

 
 
O cursor contido nesta procedure  o mais usual. O cursor de tipo pr-declarado


create or replace procedure prc_tst_cursor is

v_dt_inicio timestamp;
v_dt_fim    timestamp;

cursor cr_pessoa is
select *
  from tb_pessoa;

v_pessoa cr_pessoa%rowtype;

begin

v_dt_inicio := systimestamp;

  open cr_pessoa;
   loop
   fetch cr_pessoa into v_pessoa;
   exit when cr_pessoa%notfound;
     null; -- regras de negocio
   end loop;
  close cr_pessoa;

v_dt_fim := systimestamp;

dbms_output.put_line('Tempo processamento: '||to_char(v_dt_fim - v_dt_inicio,'dd/mm/yyyy hh24:mi:ss.ff'));

end;


Esta procedure abaixo contm uma forma de cursor bastante utilizada tambm. 
O cursor do tipo for  um dos de mais fcil de utilizao.


create or replace procedure prc_tst_for is

v_dt_inicio timestamp;
v_dt_fim    timestamp;

begin

v_dt_inicio := systimestamp;

  for dd in (select p.idpessoa
               from tb_pessoa p) loop

     null;

   end loop;

v_dt_fim := systimestamp;

dbms_output.put_line('Tempo processamento: '||to_char(v_dt_fim - v_dt_inicio,'dd/mm/yyyy hh24:mi:ss.ff'));

end;

Nesta procedure abaixo, temos a extrao da informao atravs de um bulk collection. 
Forma menos usual, alguns front end no suportam este tipo de cursor, que  o caso do FORMS.

create or replace procedure prc_tst_bulk_count is

v_dt_inicio timestamp;
v_dt_fim    timestamp;

type type_pessoa is table of tb_pessoa%rowtype;
v_tb_pessoa type_pessoa;

begin

v_dt_inicio := systimestamp;

  select *
    bulk collect into v_tb_pessoa
    from tb_pessoa;

   for dd in 1..v_tb_pessoa.count loop

     null;

   end loop;


v_dt_fim := systimestamp;

dbms_output.put_line('Tempo processamento: '||to_char(v_dt_fim - v_dt_inicio,'dd/mm/yyyy hh24:mi:ss.ff'));

end;


Nesta procedure abaixo, temos a variao do cursor de bulk collection, onde a diferena est apenas na execuo 
do loop.


create or replace procedure prc_tst_bulk_first_last is

v_dt_inicio timestamp;
v_dt_fim    timestamp;

type typ_pessoa is table of tb_pessoa%rowtype;
v_tb_pessoa typ_pessoa;

begin

v_dt_inicio := systimestamp;

  select *
    bulk collect into v_tb_pessoa
    from tb_pessoa;

   for dd in v_tb_pessoa.first.. v_tb_pessoa.last loop

     null;

   end loop;


v_dt_fim := systimestamp;

dbms_output.put_line('Tempo processamento: '||to_char(v_dt_fim - v_dt_inicio,'dd/mm/yyyy hh24:mi:ss.ff'));

end;








