/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 10 - Usando Packages.

Um package  uma coleo de objetos PL/SQL agrupados logicamente sob o nome de pacote. Os packages podem incluir 
procedures, functions, cursores, declaraes, tipos e tambm variveis??. H inmeras vantagens em coletar objetos 
em um package. Mais a frente sero vistos os benefcios de sua utilizao.

Os packages geralmente possuem duas partes, uma que se refere a spec e a segunda parte que  o body. A spec  o que 
chamamos de interface para o package.  nela que declaramos os tipos, variveis, constantes, excees, cursores e 
subprogramas que podero ser referenciados a partir de partes externas ao package. Por outro lado, no body definimos 
as consultas para os cursores e o cdigo para os subprogramas. Em alguns casos pode ser que no haja a necessidade de 
se ter um body.

Em outras palavras, Package PL/SQL  um grupo de funes relacionadas, procedimentos, tipos, cursores, dentre outros. 
Os packages so como uma biblioteca, pois uma vez escrito  armazenado no banco de dados Oracle e pode ser usado por 
muitas aplicaes.


Quais as vantagens de utilizao de packages PL/SQL?
A utilizao de packages tem uma longa histria no que diz respeito a engenharia de softwares, oferecendo caractersticas 
importantes de fcil manuteno, reutilizao de cdigo, muitas vezes em esforos dentro de uma equipe de desenvolvimento 
para o desenvolvimento de grandes sistemas. Apresentaremos ento alguns dos principais pontos vantajosos na utilizao 
de packages.

Modularidade - Packages permitem encapsular tipos relacionados logicamente, itens e sub-programas em um mdulo PL/SQL. 
Cada package fica fcil de entender, e as interfaces entre eles tornam-se mais simples, claras e bem definidas. 
Isso ajuda e muito no desenvolver de aplicativos.

Ocultao de informaes - Com os packages, podemos especificar quais tipos, itens e sub-programas sero pblicos 
(visveis e acessveis) ou privados (escondido e inacessvel). Por exemplo, se um pacote contm quatro subprogramas, 
trs podero ser pblicos e um deles privado. O package esconde a implementao de subprogramas privados para que 
somente o prprio package (e no a sua aplicao) seja afetado se houver mudanas de implementao. Isso simplifica a 
manuteno e o aprimoramento no desenvolvimento. Alm disso, ao esconder os detalhes da implementao dos usurios, 
isso remete a proteger a integridade dos packages.

Melhor performance - Quando chamamos um subprograma empacotado pela primeira vez, todo o package  carregado na memria. 
O que quer dizer que quando se tem a necessidade de reutilizao do package, ele ser recuperado de maneira mais rpida
e eficiente por estar em memria, no havendo a necessidade de busca no disco.


-- criando uma package
create or replace PACKAGE HR.PKG_EMPLOYEES IS
    FUNCTION FC_VALOR_COMISSAO (P_EMPLOYEEID NUMBER) RETURN NUMBER;
    PROCEDURE SP_EMPREGADO_S(P_CR OUT SYS_REFCURSOR); -- varivel p_cr ser de um tipo cursor dentro de uma procedure
END;

-- corpo da package
CREATE OR REPLACE PACKAGE BODY HR.PKG_EMPLOYEES IS
    FUNCTION FC_VALOR_COMISSAO (P_EMPLOYEEID NUMBER) RETURN NUMBER IS
        V_VALORCOMISSAO NUMBER;
    BEGIN
        SELECT  ROUND(NVL(COMMISSION_PCT,0) * SALARY,2) INTO V_VALORCOMISSAO
        FROM    HR.EMPLOYEES
        WHERE   EMPLOYEE_ID = P_EMPLOYEEID;
    
        RETURN V_VALORCOMISSAO;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            V_VALORCOMISSAO := -1;
    END FC_VALOR_COMISSAO;  
    
    PROCEDURE SP_EMPREGADO_S(P_CR OUT SYS_REFCURSOR)
    IS
    BEGIN
      OPEN P_CR FOR
        SELECT FIRST_NAME, LAST_NAME FROM HR.EMPLOYEES;  
    END SP_EMPREGADO_S;    
END;

-- chamando objetos
-- 1: chamando a SP HR.SP_EMPREGADO_S dentro da package PKG_EMPLOYEES
SET SERVEROUTPUT ON
DECLARE
    V_CURSOR  SYS_REFCURSOR;
    V_FN   HR.EMPLOYEES.FIRST_NAME%TYPE;
    V_LN   HR.EMPLOYEES.LAST_NAME%TYPE;    
BEGIN
    HR.PKG_EMPLOYEES.SP_EMPREGADO_S (V_CURSOR);
    LOOP 
      FETCH V_CURSOR INTO V_FN, V_LN;
      EXIT WHEN V_CURSOR%NOTFOUND;
      DBMS_OUTPUT.PUT_LINE(V_FN || ' ' || V_LN);
    END LOOP;
    CLOSE V_CURSOR;
END;

-- 2: chamando a funcao FC_VALOR_COMISSAO dentro da package PKG_EMPLOYEES
SELECT HR.PKG_EMPLOYEES.FC_VALOR_COMISSAO(151) FROM DUAL;



-- sobrecarga de packages
-----/*Sobregarga*/-----
CREATE OR REPLACE PACKAGE HR.STD
IS
    FUNCTION TO_CHAR_AULA(p_valor DATE) RETURN VARCHAR2;
    FUNCTION TO_CHAR_AULA(p_valor NUMBER) RETURN VARCHAR2;
END;
/
--TIPO DE DADO E/OU QUANTIDADE TEM QUE SER DIFERENTE PARA A SOBRECARGA FUNCIONAR
CREATE OR REPLACE PACKAGE BODY HR.STD
IS
    FUNCTION TO_CHAR_AULA(p_valor DATE) RETURN VARCHAR2
    IS
    BEGIN
      RETURN TO_CHAR(p_valor, 'dd/mm/yyyy hh24:mi:ss' );
    END;
    
    FUNCTION TO_CHAR_AULA(p_valor NUMBER) RETURN VARCHAR2
    IS
    BEGIN      
      RETURN TO_CHAR(p_valor + 1);
    END;
END;
/

-- chamando as funcoes sobrecarregadas que acabamos de criar - a function identifica o tipo e escolhe qual usar
SELECT HR.STD.TO_CHAR_AULA(SYSDATE) FROM DUAL;
SELECT HR.STD.TO_CHAR_AULA(1) FROM DUAL;

-- chamando funcao TO_CHAR do BD
SELECT TO_CHAR(SYSDATE) FROM DUAL;


-----/*FORWARD DECLARATIONS*/----- conseguirmos esconder codigos 
CREATE OR REPLACE PACKAGE HR.STD
IS
    FUNCTION TO_CHAR_AULA(p_valor DATE) RETURN VARCHAR2;
    FUNCTION TO_CHAR_AULA(p_valor NUMBER) RETURN VARCHAR2;
END;
/

CREATE OR REPLACE PACKAGE BODY HR.STD
IS
    --FUNCAO INVISIVEL EXTERNAMENTE
    FUNCTION FORMATA_DATA(p_valor DATE) RETURN VARCHAR2 
    IS 
    BEGIN
      RETURN TO_CHAR(p_valor, 'dd/mm/yyyy hh24:mi:ss');
    END;
    
    FUNCTION TO_CHAR_AULA(p_valor DATE) RETURN VARCHAR2
    IS
    BEGIN
      RETURN FORMATA_DATA(p_valor); --ESTA FUNCAO TEM QUE ESTAR DECLARADA ANTES
    END;
    
    FUNCTION TO_CHAR_AULA(p_valor NUMBER) RETURN VARCHAR2
    IS
    BEGIN      
      RETURN TO_CHAR(p_valor + 1);
    END;
END;
/

-- a funcao pode estar na especificao mas se nao tiver no corpo ela nao existe.

SELECT HR.STD.TO_CHAR_AULA(SYSDATE) FROM DUAL;
SELECT HR.STD.TO_CHAR_AULA(1) FROM DUAL;
SELECT HR.STD.FORMATA_DATA(SYSDATE) FROM DUAL;


-- compartilhamento de valores variveis
-----/**/------  compartilhar valores -- 
CREATE USER BASECOMUM IDENTIFIED BY basecomum ACCOUNT LOCK ;

CREATE OR REPLACE PACKAGE hr.PKG_TAXAS
IS
    V_INSS NUMBER :=11; -- durante o processamento pode alterar mas depois o valor retorna novamente
    V_FGTS NUMBER :=8;    
    FUNCTION RETORNA_INSS RETURN NUMBER;
    FUNCTION ALTERA_INSS (p_valor NUMBER) RETURN BOOLEAN;
END;
/

CREATE OR REPLACE PACKAGE body hr.PKG_TAXAS
IS
    FUNCTION RETORNA_INSS RETURN NUMBER
    IS 
    BEGIN
        RETURN V_INSS;
    END RETORNA_INSS;
    
    FUNCTION ALTERA_INSS (p_valor NUMBER) RETURN BOOLEAN
    IS 
    BEGIN
        V_INSS:=P_VALOR;
        return true;
    END ALTERA_INSS;
END;
/

-----/*BLOCO DE INICIALIZACAO COM VARIVEIS*/-----
CREATE OR REPLACE PACKAGE HR.STD
IS
    FUNCTION TO_CHAR_AULA(p_valor DATE) RETURN VARCHAR2;
    FUNCTION TO_CHAR_AULA(p_valor NUMBER) RETURN VARCHAR2;
    V_PI NUMBER;
END;
/
-- valor armazenado para o PI fica no select abaixo no fim da package
CREATE OR REPLACE PACKAGE BODY HR.STD
IS  
    FUNCTION FORMATA_DATA(p_valor DATE) RETURN VARCHAR2 
    IS 
    BEGIN
      RETURN TO_CHAR(p_valor, 'dd/mm/yyyy hh24:mi:ss');
    END;
    
    FUNCTION TO_CHAR_AULA(p_valor DATE) RETURN VARCHAR2
    IS
    BEGIN
      RETURN FORMATA_DATA(p_valor);
    END;
   
    FUNCTION TO_CHAR_AULA(p_valor NUMBER) RETURN VARCHAR2
    IS
    BEGIN      
      RETURN TO_CHAR(p_valor + V_PI);
    END;
    
    --BLOCO DE INICIALIZACAO
    BEGIN
      SELECT    3.14 INTO V_PI
      FROM      DUAL;
END;
/

SELECT HR.STD.TO_CHAR_AULA(1) FROM DUAL;


-- Wrapping
-- Esconder o cdigo fonte

Diretrizes para Embrulho (Wrapped)

:: Escone apenas o corpo de um tipo de pacote ou objeto, no a especificao.
:: Isso permite que outros desenvolvedores para ver as informaes que eles devem usar o pacote ou tipo, 
   mas impede de ver a sua implementao.
:: Esconde cdigo s depois de ter terminado a edit-lo.
:: Para alterar Wrapped cdigo PL / SQL, edite o arquivo fonte original e, em seguida, Wrapped novamente.
:: Antes de distribuir um arquivo envolvido, v-lo em um editor de texto para ter certeza de que todas as peas 
importantes so envolvidos.

    Package specification
    Package body
    Type specification
    Type body
    Function
    Procedure
    
Limitaes de Wrapped

O Wrapped no  um mtodo seguro para esconder senhas ou nomes de tabela.
Wrapped no esconde o cdigo fonte para os gatilhos.
Para ocultar o funcionamento de um gatilho, escrever um gatilho de uma linha que chama um subprograma com Wrapped
O Wrapped no detecta erros de sintaxe ou de semntica.
Embrulho detecta apenas os erros oracle mas no de tabelas inexistentes ou views. 
Pode ocorrer problemas entre verses de banco de dados

CREATE or REPLACE PROCEDURE hr.wrap_it (seed_in NUMBER)
    IS
      v_rand INTEGER;
    BEGIN
      DBMS_RANDOM.INITIALIZE (seed_in);
      FOR i IN 1..5 LOOP
       v_rand := mod(abs(DBMS_RANDOM.RANDOM),45);
       dbms_output.put_line(i||': '||v_rand);
      END LOOP;
  END;

SET SERVEROUTPUT ON    
execute hr.wrap_it(12);

WRAP INAME=02_Package_body.sql

https://docs.oracle.com/cd/B10501_01/appdev.920/a96624/c_wrap.htm


DROP PACKAGE HR.PKG_EMPLOYEES;
DROP PACKAGE BODY HR.PKG_EMPLOYEES;


SELECT  OWNER, OBJECT_NAME
FROM    ALL_OBJECTS
WHERE   OBJECT_TYPE = 'PACKAGE';

SELECT  OWNER, OBJECT_NAME
FROM    ALL_OBJECTS
WHERE   OBJECT_TYPE = 'PACKAGE BODY';


SELECT  *
FROM    ALL_SOURCE
WHERE   NAME = 'STD'
AND     TYPE = 'PACKAGE';

SELECT  TEXT
FROM    ALL_SOURCE
WHERE   NAME ='STD'
AND     TYPE = 'PACKAGE BODY';














