/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/

Captulo 11 - Triggers.

Um trigger  usado para escrever a lgica de procedimento que  invocada em resposta a um evento especfico. A aplicao criativa
dos triggers de banco de dados permite que voc realize muitas coisas teis que, de outra forma, no seriam possveis.

Os exemplos daquilo que voc pode fazer com os triggers incluem a replicao de dados, o armazenamento dos dados de forma 
redundante para evitar associaes freqentes de tabela e a implantao de regras de negcios complexas.

O que  um trigger?

Um trigger  um bloco PL/SQL que  associado a um evento especfico, armazenado em um banco de dados e executado sempre que o evento ocorrer.

Oracle agora suporta quatro tipos fundamentais de triggers:

Os triggers DML so os triggers tradicionais INSERT, UPDATE e DELETE que a Oracle tem suportado h anos.

Os triggers instead-of foram introduzidos com a Oracle como um modo de possibilitar a atualizao
de determinados tipos de vises. Os triggers DDL e os triggers de evento de banco de dados so novos na Oracle8i.

Triggers DML

Os triggers DML so os triggers tradicionais que podem ser definidos em uma tabela e so executados,
ou disparados, em resposta aos seguintes eventos: 

Uma linha de uma tabela  atualizada. Uma linha de uma tabela  excluda. No  possvel definir um trigger para ser 
disparado quando uma linha  selecionada.

Uma definio de trigger DML consiste nestas partes bsicas:

  O evento que dispara o trigger. 
  A tabela do banco de dados no qual o evento deve ocorrer.

Uma condio opcional que controla a hora em que o trigger  executado.

Um bloco PL/SQL contendo o cdigo a ser executado quando o trigger  disparado, ou uma declarao
CALL para um procedimento armazenado.

Um trigger  um objeto de banco de dados, assim como uma tabela ou um ndice.

Quando voc define um trigger, ele se torna parte do banco de dados e  sempre executado quando o evento
para o qual ele  definido ocorre.

No importa se o evento  disparado por alguma digitao em uma declarao SQL usando o SQL Plus, executando um 
programa de cliente-servidor que atualiza o banco de dados, ou executam um utilitrio
como o SQL"-Loader do Oracle para fazer o bulk-load dos dados. 

Por causa disso, um trigger permite que voc execute validao crtica ou clculos em resposta s alteraes feitas no banco
de dados, seja qual for a fonte.

A partir do Oracle 11g  possvel controlar a sequencia de disparo da trigger, isso  quem vai ser executado primeiro.

A partir do Oracle 8i  possvel criar mais de uma trigger do mesmo tipo na mesma tabela. Exemplo: duas triggers 
AFTER EACH ROW on INSERT. O tipo de trigger determina a ordem de execuo: As BEFORE so executadas antes, e as AFTER 
depois. Contudo, como saber qual trigger executar primeiro quando se tem duas AFTER EACH ROW on INSERT ?
A execuo das triggers neste caso seria algo randomico, pois no havia uma forma de definir a ordem de execuo.

create or replace trigger tr_pay_follow_up
before update
on payments
for each row    
FOLLOWS TRIGGER_ANTERIOR
begin
.. comandos da trigger
end;

Neste caso a clusula FOLLOWS fora que a trigger seja disparada apenas aps a trigger citada.

Existem riscos com as triggers quando uma trigger dispara outra triggers e assim causar um loop entre elas.
O Oracle 11g e as verses mais rescentes limitam a 32 o numero de triggers em cascata.
Outra restrio da trigger  que seu corpo no pode conter mais do que 32.760 bites, isso acontece porque corpo de triggers
so armazenadas em LONG
 preciso ter o privilgio CREATE TRIGGER e como alternativa ter os privilgios ALTER ANY TABLE e CREATE ANY TRIGGER

Tipos de triggers :
1 -Linguagem de definio de triggers : disparam quando voc cria, altera ou remove  objetos de um esquema
2 -DML - disparam quando voce insere atualiza ou deleta dados em uma tabela podendo disparar uma vez para toda ao
         na tabela ou somente uma por linha
3 - Triggers Compostas : Insere, deleta e altera registros em tabelas, usadas para revisar salvar ou substituir valores antes
                         que sejam alterados.  Chamadas de triggeres de captura de informaes
4 - Instead-of triggers : Geralmente usadas para gerar escritas em views no atualizaveis
5 - Sistema ou Eventos de triggeres em banco de dados : disparam quando uma atividade no banco ocorre, exemplo create user.


Um exemplo de trigger DML 

1 - criando uma simples trigger de auditoria 

drop table hr.func

drop table hr.func_aud

create table hr.func (id number, nome varchar(50), salario number);
create table hr.func_aud (id number, nome varchar(50), salario number, operacao varchar(50), usuario varchar(50));


drop trigger hr.ins_func
create or replace trigger hr.ins_func after insert on hr.func
REFERENCING NEW AS NEW OLD AS OLD
for each row
begin
  insert into hr.func_aud  values (:new.id, :new.nome, :new.salario, 'INSERT', (sys_context('USERENV', 'SESSION_USER'))); -- sysdate
end;

insert into  hr.func values (1, 'PEDRO CARVALHO', 5000);
insert into  hr.func values (1, 'JORGE ADAO', 500);
insert into  hr.func values (1, 'MARCELO RIBEIRO', 7000);
insert into  hr.func values (1, 'GUILHERME BORGES', 8000);
insert into  hr.func values (1, 'HENRIQUE BORGES', 9500);
commit;

select * from hr.func
select * from hr.func_aud

drop trigger hr.ins_upd
create or replace trigger hr.ins_upd after update on hr.func
REFERENCING NEW AS NEW OLD AS OLD
for each row
begin
  update hr.func_aud set salario = :new.salario,
                         operacao = 'UPDATE salrio anterior  ' || '-' || :old.salario || '-' || :new.salario || '-' ||:new.nome
  where nome = :new.nome;
end;


update hr.func set salario = 1
  where nome = 'JORGE ADAO';
  commit;
  
select * from hr.func

select * from hr.func_aud

drop trigger hr.ins_del
create or replace trigger hr.ins_del after delete on hr.func
REFERENCING NEW AS NEW OLD AS OLD
for each row
begin
  insert into hr.func_aud  values (:old.id, :old.nome, :old.salario, 'DELETE', (sys_context('USERENV', 'SESSION_USER')));
end;

delete from hr.func
where nome = 'JORGE ADAO';
commit;

select * from hr.func

select * from hr.func_aud



create or replace trigger hr.ins_autitoria after insert or update or delete on hr.func
REFERENCING NEW AS NEW OLD AS OLD
for each row
begin
      IF INSERTING THEN
        insert into hr.func_aud  values (:new.id, :new.nome, :new.salario, 'INSERT', (sys_context('USERENV', 'SESSION_USER'))); -- sysdate
      END IF;
      
      IF UPDATING THEN
         update hr.func_aud set salario = :new.salario,
                         operacao = 'UPDATE salrio anterior  ' || '-' || :old.salario || '-' || :new.salario || '-' ||:new.nome
         where nome = :new.nome;
      END IF;

      IF DELETING THEN
         insert into hr.func_aud  values (:old.id, :old.nome, :old.salario, 'DELETE', (sys_context('USERENV', 'SESSION_USER')));
      END IF;
end;

insert into  hr.func values (5, 'PABLO FUENTE', 1500);
commit;

select * from hr.func
select * from hr.func_aud


update hr.func set salario = 1
  where nome = 'PABLO FUENTE';
  commit;

SELECT * FROM hr.func_aud

delete from hr.func where id = 5;
COMMIT;

SELECT * FROM hr.func_aud



Note:
The INSERTING, DELETING, and UPDATING conditional predicates cannot be used for the CALL procedures; they can only be used in a PL/SQL block.

-- TRIGGERS DE EVENTOS DE BANCO LOGON

create or replace trigger HR.TR_LOGON
	AFTER LOGON ON DATABASE
WHEN (USER = 'HR')
BEGIN
	EXECUTE IMMEDIATE 'ALTER SESSION SET OPTIMIZER_MODE = ''FIRST_ROWS''';
END;

-- erro de tabela mutante
CREATE OR REPLACE TRIGGER hr.tr_emp
  AFTER DELETE ON hr.employees
    FOR EACH ROW
DECLARE
-- PRAGMA AUTONOMOUS_TRANSACTION;
  v_count INTEGER;
BEGIN
  SELECT COUNT(*) INTO v_count FROM hr.employees;
  DBMS_OUTPUT.PUT_LINE('Agora existem ' || v_count || ' empregados.');
END;

DELETE FROM hr.employees WHERE employee_id = 101;

drop TRIGGER HR.TR_EMPLOYEES_LOG;

SELECT * FROM ALL_TRIGGERS;

Maiores informaes
https://docs.oracle.com/cd/B10501_01/appdev.920/a96590/adg13trg.htm


--------------------- exemplo real trigger --------------------------

--------------------------------------------------------
--  Arquivo criado - Sbado-Novembro-01-2014   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Trigger PEDRO_CTRL_SOL_EXA_LAB_RX
--------------------------------------------------------

CREATE OR REPLACE TRIGGER "DBAMV"."PEDRO_CTRL_SOL_EXA_LAB_RX" 
 after update on DBAMV.PRE_MED  REFERENCING NEW AS NEW OLD AS OLD
for each row

declare
PRAGMA AUTONOMOUS_TRANSACTION;

  --- variaveis para lab ----
  v_qtd_dia_min_ree_lab number(4);
  v_ds_tip_presc_lab varchar(60);
  v_cd_exa_lab number(4);
  v_prontuario_lab number(8);
  v_ultimo_dt_laudo_lab date;
  v_concatena_lab varchar(1000);
  v_total_dias_lab number;
  v_cd_atendimento_lab number(8);
  v_cd_tip_presc_lab number(8);
  v_cd_tip_presc_pode_lab number;
  v_cd_paciente_pode_lab number;
  v_nm_usuario_lab varchar(50);
  v_cd_atendimento_lab_pode number;
  v_cd_pre_med_lab number;  
  
  --- variaveis para rx ----
  v_qtd_dia_min_ree_rx number(4);
  v_ds_tip_presc_rx varchar(60);
  v_cd_exa_rx number(4);
  v_prontuario_rx number(8);
  v_ultimo_dt_laudo_rx date;
  v_concatena_rx varchar(1000);
  v_total_dias_rx number;
  v_cd_atendimento_rx number(8);
  v_cd_tip_presc_rx number(8);
  v_cd_tip_presc_pode_rx number;
  v_cd_paciente_pode_rx number;
  v_nm_usuario_rx varchar(50);
  v_cd_atendimento_rx_pode number;
  v_cd_pre_med_rx number;  
  
  
  ----- globais ---------
  v_flg_aud CHAR(1);
  v_flg_sta_lab char(1);
  v_flg_sta_rx char(1);
  v_chama_erro char(1);
  erro exception;

----------------------- cursores habilita/desabilita trigger ----------------------------------------
cursor ativa_aud is
   select FLG_LIG_AUD_EXA from progress.hbgere_par;  
   
----------------------- cursores verifica documento de pronturio ----------------------------------------
cursor pode_lab is -- estudar possibilidade do mesmo mdico pedir em atendimentos diferentes
   select flg_sta, cd_tip_presc, cd_paciente, nm_usuario, cd_atendimento, cd_pre_med from dbamv.hbpsdi_jus_sol_exa
    where cd_paciente = v_prontuario_lab
      and cd_tip_presc = v_cd_tip_presc_lab
        and cd_atendimento = :new.cd_atendimento
         --and flg_sta = 'A'
           and nm_usuario = sys_context('USERENV', 'SESSION_USER') --sys_context('USERENV', 'SESSION_USER')
            and cd_pre_med = :new.cd_pre_med
            and dat_inc > sysdate - v_qtd_dia_min_ree_lab;
            
----------------------- cursores verifica documento de pronturio ----------------------------------------
cursor pode_rx is
   select flg_sta, cd_tip_presc, cd_paciente, nm_usuario, cd_atendimento, cd_pre_med from dbamv.hbpsdi_jus_sol_exa
    where cd_paciente = v_prontuario_rx
      and cd_tip_presc = v_cd_tip_presc_rx
        and cd_atendimento = :new.cd_atendimento
         --and flg_sta = 'A'
           and nm_usuario = sys_context('USERENV', 'SESSION_USER')
            and cd_pre_med = :new.cd_pre_med
              and dat_inc > sysdate - v_qtd_dia_min_ree_rx;

------------------------cursores para exames de lab-------------------------------      
cursor itens_prescritos_lab is  
      select distinct trunc(progress.hbpssd_exa_par.qtd_dia_min_ree,0),   
            dbamv.tip_presc.ds_tip_presc, 
            dbamv.tip_presc.cd_exa_lab, 
            dbamv.paciente.cd_paciente,
            dbamv.atendime.cd_atendimento,
            dbamv.tip_presc.cd_tip_presc
      from  dbamv.paciente,
            dbamv.atendime, 
            dbamv.pre_med,
            dbamv.itpre_med,
            dbamv.tip_presc,
            dbamv.exa_lab,
            progress.hbpssd_exa_par,  
            PROGRESS.HBPSSD_TIP_SET,
            PROGRESS.HBPSSD_TIP_SET_SET 
      where dbamv.paciente.cd_paciente            = dbamv.atendime.cd_paciente
      and   dbamv.atendime.cd_atendimento      = dbamv.pre_med.cd_atendimento
      and   dbamv.pre_med.cd_pre_med              = dbamv.itpre_med.cd_pre_med
      and   dbamv.tip_presc.cd_tip_presc          = dbamv.itpre_med.cd_tip_presc
      and   dbamv.tip_presc.cd_exa_lab            = dbamv.exa_lab.cd_exa_lab
      and   progress.hbpssd_exa_par.cd_exa        = dbamv.tip_presc.cd_exa_lab
      and   progress.hbpssd_exa_par.cod_tip_set   = PROGRESS.HBPSSD_TIP_SET.cod_tip_set     
      and   PROGRESS.HBPSSD_TIP_SET.cod_tip_set   = PROGRESS.HBPSSD_TIP_SET_SET.cod_tip_set
      and   progress.hbpssd_exa_par.flg_ati       = 'S'
      and   progress.hbpssd_exa_par.flg_blo_pre   = 'S'
      and   dbamv.tip_presc.sn_ativo              = 'S'
      and   (dbamv.itpre_med.sn_cancelado          = 'N' or  dbamv.itpre_med.sn_cancelado is null)  
      and   dbamv.tip_presc.cd_tip_esq            = 'LAB'
      and   dbamv.pre_med.cd_pre_med              = :new.cd_pre_med --5280769
      and   PROGRESS.HBPSSD_TIP_SET_SET.cd_setor  = :new.cd_setor --170
      and   dbamv.atendime.cd_convenio in (1,2);
      
  
cursor ultima_prescricao_lab is
    select * from (
        select  trunc(SYSDATE - dbamv.itped_lab.dt_laudo)
        from  dbamv.paciente,
              dbamv.atendime,
              dbamv.ped_lab,
              dbamv.itped_lab 
        where dbamv.paciente.cd_paciente     = dbamv.atendime.cd_paciente
        and   dbamv.ped_lab.cd_atendimento   = dbamv.atendime.cd_atendimento
        and   dbamv.itped_lab.cd_ped_lab     = dbamv.ped_lab.cd_ped_lab
        and   dbamv.itped_lab.cd_exa_lab     = v_cd_exa_lab 
        and   dbamv.paciente.cd_paciente     = v_prontuario_lab
        and   dbamv.itped_lab.sn_REALIZADO   = 'S'
        and   dbamv.itped_lab.dt_laudo       is not null
        order by  dbamv.itped_lab.dt_laudo desc)
        where rownum <=1;



------------------------cursores para exames de rx-------------------------------      
cursor itens_prescritos_rx is  
      select distinct trunc(progress.hbpssd_exa_par.qtd_dia_min_ree,0),   
            dbamv.tip_presc.ds_tip_presc, 
            dbamv.tip_presc.cd_exa_rx, 
            dbamv.paciente.cd_paciente,
            dbamv.atendime.cd_atendimento,
            dbamv.tip_presc.cd_tip_presc
      from  dbamv.paciente,
            dbamv.atendime, 
            dbamv.pre_med,
            dbamv.itpre_med,
            dbamv.tip_presc,
            dbamv.exa_rx,
            progress.hbpssd_exa_par,  
            PROGRESS.HBPSSD_TIP_SET,
            PROGRESS.HBPSSD_TIP_SET_SET 
      where dbamv.paciente.cd_paciente            = dbamv.atendime.cd_paciente
      and   dbamv.atendime.cd_atendimento      = dbamv.pre_med.cd_atendimento
      and   dbamv.pre_med.cd_pre_med              = dbamv.itpre_med.cd_pre_med
      and   dbamv.tip_presc.cd_tip_presc          = dbamv.itpre_med.cd_tip_presc
      and   dbamv.tip_presc.cd_exa_rx             = dbamv.exa_rx.cd_exa_rx
      and   progress.hbpssd_exa_par.cd_exa        = dbamv.tip_presc.cd_exa_rx
      and   progress.hbpssd_exa_par.cod_tip_set   = PROGRESS.HBPSSD_TIP_SET.cod_tip_set     
      and   PROGRESS.HBPSSD_TIP_SET.cod_tip_set   = PROGRESS.HBPSSD_TIP_SET_SET.cod_tip_set
      and   progress.hbpssd_exa_par.flg_ati       = 'S'
      and   progress.hbpssd_exa_par.flg_blo_pre   = 'S'
      and   dbamv.tip_presc.sn_ativo              = 'S'
      and   (dbamv.itpre_med.sn_cancelado          = 'N' or  dbamv.itpre_med.sn_cancelado is null)  
      and   dbamv.tip_presc.cd_tip_esq            in ('RX','EXA')
      and   dbamv.pre_med.cd_pre_med              = :new.cd_pre_med
      and   PROGRESS.HBPSSD_TIP_SET_SET.cd_setor  = :new.cd_setor
      and   dbamv.atendime.cd_convenio in (1,2);
      
    
cursor ultima_prescricao_rx is
/*    select * from (
        select  trunc(SYSDATE - dbamv.itped_rx.dt_realizado)
        from  dbamv.paciente,
              dbamv.atendime,
              dbamv.ped_rx,
              dbamv.itped_rx 
        where dbamv.paciente.cd_paciente    = dbamv.atendime.cd_paciente
        and   dbamv.ped_rx.cd_atendimento   = dbamv.atendime.cd_atendimento
        and   dbamv.itped_rx.cd_ped_rx      = dbamv.ped_rx.cd_ped_rx
        and   dbamv.itped_rx.cd_exa_rx      = v_cd_exa_rx 
        and   dbamv.paciente.cd_paciente    = v_prontuario_rx
        and   dbamv.itped_rx.sn_REALIZADO   = 'S'
        and   dbamv.itped_rx.dt_realizado   is not null
        order by  dbamv.itped_rx.dt_realizado desc)
        where rownum <=1;*/
        
    select * from (
        select trunc(SYSDATE - dbamv.pre_med.dt_pre_med)
        from  dbamv.paciente,
              dbamv.atendime,
              dbamv.pre_med,
              dbamv.itpre_med 
        where dbamv.paciente.cd_paciente    = dbamv.atendime.cd_paciente
        and   dbamv.pre_med.cd_atendimento  = dbamv.atendime.cd_atendimento
        and   dbamv.pre_med.cd_pre_med      = dbamv.itpre_med.cd_pre_med
        and   dbamv.itpre_med.cd_tip_presc  = v_cd_tip_presc_rx 
        and   dbamv.paciente.cd_paciente    = v_prontuario_rx
        and   (dbamv.itpre_med.sn_cancelado <> 'S')  
        and   dbamv.pre_med.cd_pre_med <> :new.cd_pre_med
        order by  dbamv.pre_med.dt_pre_med desc)
        where rownum <=1;
        
        --5484	EXA	TC CRANIO		1058 1943710
        
begin
  v_chama_erro := 'N';
   
   open ativa_aud;
   loop
     fetch ativa_aud into v_flg_aud;  
     exit when ativa_aud%notfound;
   end loop;  
   close ativa_aud;
 
--if (sys_context('USERENV', 'SESSION_USER') = 'PEDRO.CARVALHO') or (sys_context('USERENV', 'SESSION_USER') = 'FLAVIO.BARDELLA') THEN   

   if (:new.fl_impresso = 'S') and (v_flg_aud = 'S') and SYS_CONTEXT( 'USERENV', 'ACTION' ) = 'M_PREMED' then -- se foi impresso e se a trigger esta como SIM para execuo   
     
             open itens_prescritos_lab;
             loop
                fetch itens_prescritos_lab into v_qtd_dia_min_ree_lab, v_ds_tip_presc_lab, v_cd_exa_lab, v_prontuario_lab, v_cd_atendimento_lab, v_cd_tip_presc_lab;  
                exit when itens_prescritos_lab%notfound;
                
                open ultima_prescricao_lab;
                loop
                   fetch ultima_prescricao_lab into v_total_dias_lab;
                   exit when ultima_prescricao_lab%notfound;
                   
                   if  (v_qtd_dia_min_ree_lab > v_total_dias_lab) then
                        open pode_lab; 
                        loop
                          fetch pode_lab into v_flg_sta_lab, v_cd_tip_presc_pode_lab, v_cd_paciente_pode_lab, v_nm_usuario_lab, v_cd_atendimento_lab_pode, v_cd_pre_med_lab;  
                          exit when pode_lab%notfound;
                        end loop;       
                        close pode_lab;
                        
                        if (v_flg_sta_lab = 'A') and (v_cd_tip_presc_pode_lab = v_cd_tip_presc_lab) and (v_cd_paciente_pode_lab = v_prontuario_lab) and (v_cd_atendimento_lab_pode = :new.cd_atendimento) and (v_cd_pre_med_lab = :new.cd_pre_med)then
                            update dbamv.hbpsdi_jus_sol_exa  set flg_sta = 'F' where cd_tip_presc = v_cd_tip_presc_lab and flg_sta = 'A' and nm_usuario = sys_context('USERENV', 'SESSION_USER') and cd_paciente = v_prontuario_lab AND v_cd_pre_med_lab = :new.cd_pre_med;
                            COMMIT;
                        end if;  
                        
                        if v_cd_tip_presc_pode_lab is null then -- se nao encontrou nenhum registro de documento
                              v_concatena_lab :=  v_concatena_lab || ' ' || substr(v_ds_tip_presc_lab,1,40) || ' Min: ' || v_qtd_dia_min_ree_lab || ' dias' || chr(10);
                              v_chama_erro := 'S';
                        end if; 
                         
                        if (v_cd_pre_med_lab is not null) and (v_cd_pre_med_lab <> :new.cd_pre_med) then 
                              v_concatena_lab :=  v_concatena_lab || ' ' || substr(v_ds_tip_presc_lab,1,40) || ' Min: ' || v_qtd_dia_min_ree_lab || ' dias' || chr(10);
                              v_chama_erro := 'S';
                        end if;
                        
                   end if;
                end loop;  
                close ultima_prescricao_lab;    
                      
             end loop;  
             close itens_prescritos_lab;
             
-- abertura de cursosres para exames rx
             open itens_prescritos_rx;
             loop
               fetch itens_prescritos_rx into v_qtd_dia_min_ree_rx, v_ds_tip_presc_rx, v_cd_exa_rx, v_prontuario_rx, v_cd_atendimento_rx, v_cd_tip_presc_rx;  
               exit when itens_prescritos_rx%notfound;
                       open ultima_prescricao_rx;
                       loop
                         fetch ultima_prescricao_rx into v_total_dias_rx;
                         exit when ultima_prescricao_rx%notfound;
                         
                           if  (v_qtd_dia_min_ree_rx > v_total_dias_rx) then
                                  open pode_rx; 
                                  loop
                                    fetch pode_rx into v_flg_sta_rx, v_cd_tip_presc_pode_rx, v_cd_paciente_pode_rx, v_nm_usuario_rx, v_cd_atendimento_rx_pode, v_cd_pre_med_rx;  
                                    exit when pode_rx%notfound;
                                  end loop;       
                                  close pode_rx;
  
                                    if (v_flg_sta_rx = 'A') and (v_cd_tip_presc_pode_rx = v_cd_tip_presc_rx) and (v_cd_paciente_pode_rx = v_prontuario_rx) and (v_cd_atendimento_rx_pode = :new.cd_atendimento) and (v_cd_pre_med_rx = :new.cd_pre_med)then
                                          update dbamv.hbpsdi_jus_sol_exa  set flg_sta = 'F' where cd_tip_presc = v_cd_tip_presc_rx and flg_sta = 'A' and nm_usuario = sys_context('USERENV', 'SESSION_USER') and cd_paciente = v_prontuario_rx AND v_cd_pre_med_rx = :new.cd_pre_med;
                                          commit; 
                                    end if;
                                    
                                    if v_cd_tip_presc_pode_rx is null then -- se nao encontrou nenhum registro de documento
                                         v_concatena_rx :=  v_concatena_rx || ' ' || substr(v_ds_tip_presc_rx,1,40) || ' Min: ' || v_qtd_dia_min_ree_rx || ' dias' || chr(10);
                                         v_chama_erro := 'S';
                                    end if;
                                    
                                    if (v_cd_pre_med_rx is not null) and (v_cd_pre_med_rx <> :new.cd_pre_med) then 
                                        v_concatena_rx :=  v_concatena_rx || ' ' || substr(v_ds_tip_presc_rx,1,40) || ' Min: ' || v_qtd_dia_min_ree_rx || ' dias' || chr(10);
                                        v_chama_erro := 'S';
                                    end if;
                           end if;

                         end loop;  
                      close ultima_prescricao_rx;    
                      
               end loop;  
             close itens_prescritos_rx; 
  
          if v_chama_erro = 'S' then  -- diferente de F e exa exa
            raise erro;
          end if;           
  end if;  --  if (:new.fl_impresso = 'S') and (v_flg_aud = 'S') then 
  
--end if;  -- if (sys_context('USERENV', 'SESSION_USER') = 'PEDRO.CARVALHO') or (sys_context('USERENV', 'SESSION_USER') = 'FLAVIO.BARDELLA') THEN   

  EXCEPTION  WHEN erro THEN -- bloco da exception criada
       RAISE_APPLICATION_ERROR(-20034, 'CONTROLE HB DE REEXECUO DE EXAMES                            ' || chr(10) || '-----------------------------------' || chr(10) || 'Exames inferiores ao prazo mnimo : ' || chr(10) || chr(10) || v_concatena_rx || v_concatena_lab || chr(10) || chr(10) || 'SOLICITAO BLOQUEADA' || chr(10) || 'JUSTIFIQUE SOLICITAO NO DOCUMENTO DE PRONTURIO ' || chr(10) || '747 (JUSTIFICATIVA PEDIDO EXAMES BLOQUEADOS)' || chr(10) || chr(10) || 'IMPORTANTE : Aps inserir os itens, utilize o boto GRAVAR/FINALIZAR, retorne e finalize a prescrio.' );
end;
/
ALTER TRIGGER "DBAMV"."PEDRO_CTRL_SOL_EXA_LAB_RX" ENABLE;








