/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 12 - Oracle Supplied Packages. Pacotes fornecidos

-- Execute Immediate
-- Dbms_sql
-- Native Dynamic SQL
-- Dbms_output
-- Utl_file
-- Mais informaes em https://docs.oracle.com/cd/F49540_01/DOC/server.815/a68001/intro.htm

A declarao EXECUTE IMMEDIATE executa uma instruo SQL dinmica ou annimo bloco PL / SQL. Voc pode us-lo para 
emitir instrues SQL que no podem ser representados diretamente no PL / SQL, ou para construir declaraes em que 
voc no sabe todos os nomes de tabelas, clusulas WHERE, e assim por diante com antecedncia. 

Sql Dinamico

O SQL dinmico permite que se crie procedimentos de propsito mais geral. Por exemplo, utilizando o SQL dinmico voc 
pode escrever um procedimento que opere sobre uma tabela cujo nome s ser conhecido em tempo de execuo. Ela permite 
ainda o uso da linguagem de definio de dados (DDL), isto , permite que se crie (CREATE), altere (ALTER) e remova 
(DROP) objetos dentro de um procedimento ou bloco PL/SQL, o que no  permitido com declaraes do SQL esttico.

O SQL dinmico  implementado pela package DBMS_SQL


Usando SQL Dinmico para Chamar Stored Procedures e Functions


Voc pode tambm utilizar o SQL dinmico para chamar stored procedures e functions de dentro de seu bloco PL/SQL. Para 
isto os passos so os seguintes:

-	Chamar DBMS_SQL.OPEN_CURSOR para abrir um cursor; 
-	Se existir variveis bind, vincule-as utilizando a procedure DBMS_SQL.BIND_VARIABLE; 
-	Execute a procedure ou function utilizando DBMS_SQL.EXECUTE; 
-	Se houver variveis bind, chame DBMS_SQL.VARIABLE_VALUE para obter os valores associados a elas; 
-	Feche o cursor com a procedure DBMS_SQL.CLOSE_CURSOR.

Quando voc utiliza a package DBMS_SQL para executar uma procedure ou function as variveis passadas como parmetro so 
tratadas como sendo bind, o mesmo acontece para variveis que recebem os valores retornados por functions.


SET SERVEROUTPUT ON
BEGIN
    EXECUTE IMMEDIATE 'CREATE TABLE HR.EMP2 AS SELECT * FROM HR.EMPLOYEES';
EXCEPTION
    WHEN OTHERS THEN
        NULL;
END;

select * from hr.emp2

-- Desvantagems do execute immediate poucos recursos em relao ao dbms_sql.
-- Ex banco remoto
-- Atualmente recomendado peloa Oracle e o execute_immediate

-- immediate com into
SET SERVEROUTPUT ON
DECLARE
    V_COUNT NUMBER;
BEGIN
    BEGIN
        EXECUTE IMMEDIATE 'CREATE TABLE HR.EMP2 AS SELECT * FROM HR.EMPLOYEES';
    EXCEPTION
      WHEN OTHERS THEN
        NULL;
    END;
 
    EXECUTE IMMEDIATE 'SELECT COUNT(1) FROM HR.EMP2' INTO V_COUNT;  -- diferente o into vem fora do que declarar em procedures
    DBMS_OUTPUT.PUT_LINE('O total de linhas da tabela EMP2 eh: ' || TO_CHAR(V_COUNT));
END;


-- immediate into_using
SET SERVEROUTPUT ON
DECLARE
    V_COUNT NUMBER;
BEGIN
    EXECUTE IMMEDIATE 'SELECT COUNT(1) FROM HR.EMP2 WHERE EMPLOYEE_ID = :id' INTO V_COUNT USING 101;
    DBMS_OUTPUT.PUT_LINE('O total de linhas com ID 101 da tabela EMP2 eh: ' || TO_CHAR(V_COUNT));
END;
-- Obs.: USING eh melhor do que concatenar valores pois faz uso de variavel bind


-- 1: criando usuario TESTE e tabela TABELA1
CREATE USER TESTE IDENTIFIED BY TESTE;
GRANT CREATE TABLE TO TESTE;
GRANT UNLIMITED TABLESPACE TO TESTE;
CREATE TABLE TESTE.TABELA1 (COL1 NUMBER);

-- 2: verificando se a tabela TABELA1  foi criada
SELECT   TABLE_NAME 
FROM     ALL_TABLES
WHERE    OWNER = 'TESTE';

-- 3: criando SP que permite apagar tabelas de qq usuario , conforme parametro de entrada
CREATE OR REPLACE PROCEDURE HR.SP_APAGA_TABELAS_USUARIO(P_USUARIO VARCHAR2)
AUTHID CURRENT_USER IS
BEGIN
  FOR REC_LINHA IN  (SELECT   TABLE_NAME 
                     FROM     ALL_TABLES
                     WHERE    OWNER = P_USUARIO)
  LOOP
      EXECUTE IMMEDIATE 'DROP TABLE ' || P_USUARIO || '.' || REC_LINHA.TABLE_NAME;
  END LOOP;
END;

-- 4: chamando SP para apagar tabelas do usuario TESTE
BEGIN
    HR.SP_APAGA_TABELAS_USUARIO('TESTE');
END;

-- 5: verificando se as tabelas do usurio TESTE foram apagadas
SELECT   TABLE_NAME 
FROM     ALL_TABLES
WHERE    OWNER = 'TESTE';

-- cursoor open for
SET SERVEROUTPUT ON
DECLARE
    TYPE emp_rc IS REF CURSOR;
    emp_cur emp_rc;
    EMP_ROW HR.EMPLOYEES%ROWTYPE;
    v_sql VARCHAR2(50):='SELECT * FROM HR.EMPLOYEES';
BEGIN
    OPEN emp_cur FOR v_sql;
    LOOP
        FETCH emp_cur INTO emp_row;
        EXIT WHEN  emp_cur%NOTFOUND;
        DBMS_OUTPUT.PUT_LINE('O empregado ' || emp_row.FIRST_NAME || ' ' || emp_row.LAST_NAME || ' foi contratado em ' || TO_CHAR(emp_row.HIRE_DATE,'dd/mm/yyyy'));
    END LOOP;

    CLOSE emp_cur;
END;

O exemplo seguinte cria um procedimento (stored procedure) que realiza uma consulta em uma coluna de uma tabela, o nome 
da coluna e da tabela so passados como parmetro para o procedimento.

CREATE OR REPLACE PROCEDURE HR.SQL_DINAMICO(coluna IN VARCHAR2, tabela IN VARCHAR2) IS
      cur_handle INTEGER;
      texto VARCHAR2(80);
      temp NUMBER;
BEGIN
      cur_handle := DBMS_SQL.OPEN_CURSOR;
      DBMS_SQL.PARSE(cur_handle, 'SELECT '||coluna||' FROM '||tabela, DBMS_SQL.V7);
      DBMS_SQL.DEFINE_COLUMN(cur_handle, 1, texto, 80);
      temp := DBMS_SQL.EXECUTE(cur_handle);
      LOOP
          IF DBMS_SQL.FETCH_ROWS(cur_handle) = 0 THEN EXIT;
            ELSE DBMS_SQL.COLUMN_VALUE(cur_handle, 1, texto);
            DBMS_OUTPUT.PUT_LINE(texto);
          END IF;
      END LOOP;
      DBMS_SQL.CLOSE_CURSOR(cur_handle);
END SQL_DINAMICO;

EXECUTE HR.SQL_DINAMICO('FIRST_NAME','HR.EMPLOYEES');


-- Dbms_output

No Oracle temos vrios Pacotes (Packages) internos disponveis para suprir diversas necessidades. Um desses pacotes  o 
DBMS_OUTPUT, que nos permite enviar mensagens a partir de um bloco PL/SQL, procedures, pacotes e triggers.

O principal uso do DBMS_OUTPUT  na depurao de blocos PL/SQL, e sua procedure principal  a PUT_LINE, que adiciona a 
mensagem, passada como parmetro, ao seu buffer e adiciona tambm um caracter de nova linha no final, para que uma nova 
mensagem adicionada ao buffer fique em outra linha quando for mostrada.

Eu particularmente uso muito a procedure PUT_LINE do pacote DBMS_OUTPUT, para depurar meus blocos PL/SQL.

Segue um exemplo simples de uso.

SET SERVEROUTPUT ON
DECLARE
    V_COUNT NUMBER;
BEGIN
    BEGIN
        EXECUTE IMMEDIATE 'CREATE TABLE HR.EMP2 AS SELECT * FROM HR.EMPLOYEES';
    EXCEPTION
      WHEN OTHERS THEN
        NULL;
    END;
    EXECUTE IMMEDIATE 'SELECT COUNT(1) FROM HR.EMP2' INTO V_COUNT;  -- diferente o into vem fora do que declarar em procedures
    DBMS_OUTPUT.PUT_LINE('O total de linhas da tabela EMP2 eh: ' || TO_CHAR(V_COUNT));
END;

No SQL*Plus por default no l o que um bloco PL/SQL insere no Buffer do pacote DBMS_OUTPUT.
Para forarmos o SQL*Plus a ler o Buffer temos que executar o comando set serveroutput on quando acessamos o SQL*Plus.
Mas esse Buffer do pacote DBMS_OUTPUT tem um limite e caso chegue nesse limite dar um erro.

A partir da verso 10R2 esse limite foi retirado e com o comando set serveroutput on size unlimited no temos mais esse 
problema.


-- UTL File

O pacote UTL_FILE  um conjunto de componentes que tem como funo permitir o acesso ou gerao de arquivos externos ao 
banco de dados. Um exemplo seria importarmos scripts em SQL ou PL/SQL para o nosso sistema. O pacote  constitudo de 18
componentes, nos quais so distribudos da seguinte maneira: 7 Exception, 2 Function, 8 Procedure e 1 Type.

Exemplo: Roteiro para gerao de arquivo texto:

Para usar a gravao de arquivos em disco via package e/ou procedure, precisamos primeiramente verificar o parametro 
UTL_FILE_DIR que mostra em quais diretrios temos permisso de gravar arquivos.

show parameters UTL_FILE_DIR

shutdown na instancia
alter session set utl_file_dir = 'C:\TEMP\' scope=both;
startup na instancia

CREATE OR REPLACE DIRECTORY mydir AS 'c:\temp';

GRANT READ,WRITE ON DIRECTORY mydir TO sys;

DECLARE
 V_LINHA VARCHAR2(2000) := '';
 V_ARQUIVO UTL_FILE.FILE_TYPE;
BEGIN
  V_ARQUIVO := UTL_FILE.FOPEN('MYDIR', 'ORACLE4.TXT', 'W');
    FOR i in 1..100 LOOP
      V_LINHA := 'LINHA ' || i || '!!';
      UTL_FILE.PUT_LINE(V_ARQUIVO, V_LINHA);
    END LOOP;
  UTL_FILE.FCLOSE(V_ARQUIVO);
  
END;

 /*EXCEPTION
      WHEN UTL_FILE.INVALID_OPERATION THEN
               Dbms_Output.Put_Line('Operao invlida no arquivo.');
               UTL_File.Fclose(arquivo_saida);
      WHEN UTL_FILE.WRITE_ERROR THEN
               Dbms_Output.Put_Line('Erro de gravao no arquivo.');
               UTL_File.Fclose(arquivo_saida);
      WHEN UTL_FILE.INVALID_PATH THEN
               Dbms_Output.Put_Line('Diretrio invlido.');
               UTL_File.Fclose(arquivo_saida);
      WHEN UTL_FILE.INVALID_MODE THEN
               Dbms_Output.Put_Line('Modo de acesso invlido.');
               UTL_File.Fclose(arquivo_saida);
      WHEN Others THEN
               Dbms_Output.Put_Line('Problemas na gerao do arquivo.');
               UTL_File.Fclose(arquivo_saida);*/
 

DROP TABLE HR.DADOS
Create table hr.dados ( nome varchar(50))

DECLARE
    arquivo_ler                    UTL_File.File_Type;
    Linha                               Varchar2(100);
BEGIN
    arquivo_ler := UTL_FILE.FOPEN('MYDIR', 'ORACLE4.TXT', 'R');
    Loop
        UTL_File.Get_Line(arquivo_ler, Linha);
        Insert Into hr.dados Values (Linha);
        commit;
    End Loop;
    UTL_File.Fclose(arquivo_ler);
    Dbms_Output.Put_Line('Arquivo processado com sucesso.');
/*EXCEPTION
    WHEN No_data_found THEN
               UTL_File.Fclose(arquivo_ler);
               Commit;
    WHEN UTL_FILE.INVALID_PATH THEN
               Dbms_Output.Put_Line('Diretrio invlido.');
               UTL_File.Fclose(arquivo_ler);
    WHEN Others THEN
               Dbms_Output.Put_Line ('Problemas na leitura do arquivo.');
               UTL_File.Fclose(arquivo_ler);*/
END;

select * from hr.dados
delete * from hr.dados
commit;



